#!/usr/bin/env python
# -*- coding:utf-8 -*-
import socket
import threading
import tkinter as tk


class UDP():
    def __call__(self):
        return self.receive.get()

    def __init__(self, master, *args, **kwargs):
        self.binded = False

        self.error_message_enable = kwargs.get('error_message_enable', True)
        self.error = self.Error(self)

        self._port_changing = False

        self._port = int(kwargs.get('udp_port', '4000'))
        self._bufsize = int(kwargs.get('udp_bufsize', '4096'))

        self.receive = tk.StringVar(value='')

        self._sock = None
        self._start()

    def __new__(cls, master, *args, **kwargs):
        cls.master = master

        if hasattr(cls.master, 'root'):
            cls.root = cls.master.root
            if hasattr(cls.root, 'ini'):
                cls.ini = cls.root.ini

        if hasattr(cls.master, "parent"):
            cls.parent = cls.master.parent

        return super().__new__(cls)

    @property
    def bufsize(self):
        return self._bufsize
    @bufsize.setter
    def bufsize(self, value):
        self._bufsize = value

    class Error():
        def __init__(self, master):
            self.master = master
            self.root = self.master.root

            self.id = tk.IntVar(value=0)
            self.id.trace('w', self._error_changed)
            self.message = ''
            self.title = ''

        def _error_changed(self, *args):
            if self.master.error_message_enable:
                if self.id.get() > 0:
                    tk.messagebox.showwarning(self.title, self.message)

    def _start(self):
        self._sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self._sock.settimeout(0.1)
        self._socket_bind()

        self.thread = threading.Thread(
            name='udp{}{}'.format(self.port, self.binded),
            target=self._thread,
            )
        self.thread.setDaemon(True)
        self.thread.start()

    def _socket_bind(self):
        try:
            # host is ''.
            self._sock.bind(('', self.port))
        except Exception as e:
            if e.errno == 10048:
                title = (
                    lambda: 
                        'UDPポート異常'.format(self.port)
                    if mhfn.lang() == 'ja_JP' else 
                        'Failed UDP port'.format(self.port)
                    )()
                msg = (
                    lambda: 
                        'ポート {} は使用されています'.format(self.port)
                    if mhfn.lang() == 'ja_JP' else
                        'Port {} was used.'.format(self.port)
                    )()
            else:
                title = (
                    lambda: 
                        'その他のエラー'.format(self.port)
                    if mhfn.lang() == 'ja_JP' else 
                        'Other error'.format(self.port)
                    )()
                msg = (
                    lambda: 
                        'ポート {} は使用されています'.format(self.port)
                    if mhfn.lang() == 'ja_JP' else
                        'Port {} was used.'.format(self.port)
                    )()
            #tk.messagebox.showwarning(title, msg)
            self.error.message = msg
            self.error.title = title
            self.error.id.set(1)
            self.binded = False
        else:
            self.error.message = ''
            self.error.title = ''
            self.error.id.set(0)
            self.binded = True

    def _thread(self):
        while True:
            data, addr = None, None
            while self.binded:
                try:
                    if self._port_changing:
                        break
                    data, addr = self._sock.recvfrom(self._bufsize)
                    data = data.decode()
                    if (data is not None) & (addr is not None):
                        self.receive.set('{};{};{}'.format(
                            data, addr[0], addr[1]))
                        data, addr = None, None
                except Exception as e:
                    #print(e)
                    pass

            if self._port_changing:
                self._sock.close()
                del(self._sock)
                self._sock = None

                self._sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
                self._sock.settimeout(0.1)
                self._socket_bind()

                self.thread.name = 'udp{}{}'.format(self.port, self.binded)
                self._port_changing = False

    def udp_reset(self):
        if type(self._sock) == socket.socket:
            self._sock.close()
        del(self._sock)
        self._sock = None

    @property
    def port(self):
        return self._port
    @port.setter
    def port(self, value):
        value = int(value)
        if value != self._port:
            self._port = value
            self._port_changing = True
